@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008-2021 Sergey Poznyakoff
+   Copyright (C) 2008-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 
 typedef int (*dico_list_iterator_t)(void *item, void *data);
 typedef int (*dico_list_comp_t)(const void *, const void *, void *);
+typedef int (*dico_list_reduce_t) (void *, void *, void *);
 
 dico_list_t dico_list_create(void);
 void dico_list_destroy(dico_list_t *list);
@@ -72,6 +73,7 @@ void *dico_iterator_current(dico_iterator_t itr);
 dico_iterator_t dico_list_iterator(dico_list_t list);
 void dico_iterator_destroy(dico_iterator_t *ip);
 void *dico_iterator_first(dico_iterator_t ip);
+void *dico_iterator_last(dico_iterator_t ip);
 void *dico_iterator_next(dico_iterator_t ip);
 void *dico_iterator_prev(dico_iterator_t ip);
 void *dico_iterator_item(dico_iterator_t ip, size_t n);
@@ -80,4 +82,9 @@ size_t dico_iterator_position(dico_iterator_t ip);
 void dico_iterator_remove_current(dico_iterator_t ip, void **pptr);
 void dico_iterator_set_data(dico_iterator_t ip, void *data);
 
+int dico_iterator_insert_before(dico_iterator_t ip, void *data);
+int dico_iterator_insert_after(dico_iterator_t ip, void *data);
+
+int dico_list_reduce(dico_list_t list, dico_list_reduce_t fn, void *data);
+
 #endif
